@@ -44,6 +44,8 @@ case "$1" in
         #
         sqlite3 "$VAR_DIR/$USER_DB" < /usr/share/benno-web/db-update_2.0.1.sql 2>/dev/null
         #
+        perl /usr/share/benno-web/db-update_2.8.4.pl 2>/dev/null
+        #
         set -e
 
         chown $WWW_USER "$VAR_DIR/$USER_DB"
@@ -85,7 +87,7 @@ case "$1" in
             if [ -f /etc/benno/rest.secret ];then
                 cat /etc/benno/rest.secret >> /etc/benno-web/rest.conf
             else
-                cat /dev/urandom | tr -dc 'a-zA-Z0-9' | fold -w 20 | head -n 1 >> /etc/benno-web/rest.conf
+                dd if=/dev/urandom bs=1k count=1 status=none | tr -dc a-zA-Z0-9 | head -c 20 >> /etc/benno-web/rest.conf
             fi
         fi
 
