@@ -76,5 +76,5 @@ if ($USER->permission('BNO_ADMIN')) {
 
 $BennoUser->save();
 
-$App->redirect('userList');
+$App->redirect('userList',array('MSG' => 'MSG_UPDATED'));
 ?>
