@@ -60,16 +60,42 @@ class UserDA extends Persistence
     public static function FetchList ($archive)
     {
         global $App;
-        $entries = array();
+        $entries[] = '';
+        list($cid,$scid) = explode('/',$archive,2);
+        $scid || $scid = '';        // empty values must be set explicit
 
         $DB = $App->DB;
-        $sql = "SELECT * FROM user WHERE archive = '".$archive."' ORDER BY name";
-        $res = $DB->query($sql);
-        while ($row = $res->fetchRow()) {
-            $obj = new User($row['id']);
-            $obj->load();
-            if (!$obj->name) $obj->name = $obj->id;
-            array_push($entries,$obj);
+        $sql = "SELECT userid FROM container WHERE cid=? and scid=? ORDER BY userid";
+        $sth = $DB->db->prepare($sql);
+        $sth->bindValue(1,$cid);
+        $sth->bindValue(2,$scid);
+        try {
+            if($sth->execute()) {
+                while ($row = $sth->fetch()) {
+                    $obj = new User($row['userid']);
+                    $obj->load();
+                    if (!$obj->name) $obj->name = $obj->id;
+                    $entries[$obj->id] = $obj;
+                }
+            }
+        } catch (Exception $e) {
+            $App->log('Error access container table: '.$e->getMessage());
+        }
+
+        if (empty($entries)) { // deprecated data from user table
+            $sql = "SELECT * FROM user WHERE id=?";
+            $sth = $DB->db->prepare($sql);
+            foreach($userids as $uid) {
+                $sth->bindValue(1,$cid);
+                if ($sth->execute()) {
+                    while ($row = $sth->fetch()) {
+                        $obj = new User($row['id']);
+                        $obj->load();
+                        if (!$obj->name) $obj->name = $obj->id;
+                        $entries[$obj->id] = $obj;
+                    }
+                }
+            }
         }
 
         return $entries;
@@ -80,19 +106,24 @@ class UserDA extends Persistence
     {
         global $App;
         $DB = $App->DB;
-        $sql = "SELECT * FROM user WHERE id = '$this->id'";
-        $res = $DB->query($sql);
-        // read data from query
-        $row = $res->fetchRow();
+
+        $sql = 'SELECT * FROM user WHERE id = ?';
+        $sth = $DB->db->prepare($sql);
+        $sth->bindValue(1,$this->id);
+        $res = $sth->execute();
+        $row = $sth->fetch();
         $this->password = $row['password'];
         $this->name = $row['name'];
-        array_push($this->archive,$row['archive']);
         $this->role = $row['role'];
 
+        $this->_load_container();
+
         // read address data
-        $sql = "SELECT address FROM address WHERE id = '$this->id'";
-        $res = $DB->query($sql);
-        while($row = $res->fetchRow()) {
+        $sql = 'SELECT address FROM address WHERE id = ?';
+        $sth = $DB->db->prepare($sql);
+        $sth->bindValue(1,$this->id);
+        $res = $sth->execute();
+        while($row = $sth->fetch()) {
             array_push($this->addresses,$row['address']);
         }
 
@@ -228,12 +259,20 @@ class UserDA extends Persistence
 
         $DB = $App->DB;
         $DB->startTransaction();
-        $sql = "DELETE FROM address WHERE id = '$this->id'";
-        $res = $DB->query($sql);
-
-        $sql = "INSERT INTO address (id,address) VALUES (?,?)";
-        $sth = $DB->db->prepare($sql);
         try {
+            $sql = 'DELETE FROM address WHERE id = ?';
+            $sth = $DB->db->prepare($sql);
+            $sth->bindValue(1,$this->id);
+            $res = $sth->execute();
+        }
+        catch(PDOException $e){ 
+            $App->log('WARN del address: '.$e->getMessage(),2);
+        }
+
+
+        try {
+            $sql = "INSERT INTO address (id,address) VALUES (?,?)";
+            $sth = $DB->db->prepare($sql);
             foreach ($this->addresses as $address) {
                 $sth->bindValue(1,$this->id);
                 $sth->bindValue(2,$address);
@@ -241,22 +280,51 @@ class UserDA extends Persistence
             }
         }
         catch(PDOException $e){ 
-            $App->log('WARN '.$e->getMessage(),0);
+            $App->log('WARN insert address: '.$e->getMessage(),2);
         }
 
-        // remove SYSDATA
-        $containers = '';
-        foreach ($this->archive as $container) {
-            if ($container == 'SYSDATA') continue;
-            $containers .= $container.',';
-        }
-        $containers = rtrim($containers,',');
-        
-        $sql = "DELETE FROM user WHERE id = '$this->id'";
-        $res = $DB->query($sql);
-        $sql = "INSERT INTO user (id,name,password,archive,role) VALUES (?,?,?,?,?)";
-        $sth = $DB->db->prepare($sql);
         try {
+            $containers = '';
+            $sql = 'DELETE FROM container WHERE userid = ?';
+            $sth = $DB->db->prepare($sql);
+            $sth->bindValue(1,$this->id);
+            $res = $sth->execute();
+        }
+        catch(PDOException $e){ 
+            $App->log('WARN del container: '.$e->getMessage(),2);
+        }
+
+        try {
+            $sql = "INSERT INTO container (userid,cid) VALUES (?,?)";
+            $sth = $DB->db->prepare($sql);
+            foreach ($this->archive as $container) {
+                if ($container == 'SYSDATA') continue;
+                $sth->bindValue(1,$this->id);
+                $sth->bindValue(2,$container);
+
+                $res = $sth->execute();
+ 
+                $containers .= $container.','; // deprecated
+            }
+            $containers = rtrim($containers,','); // deprecated
+        }
+        catch(PDOException $e){ 
+            $App->log('WARN insert container: '.$e->getMessage(),2);
+        }
+
+        try {
+            $sql = 'DELETE FROM user WHERE id = ?';
+            $sth = $DB->db->prepare($sql);
+            $sth->bindValue(1,$this->id);
+            $res = $sth->execute();
+        }
+        catch(PDOException $e){ 
+            $App->log('WARN del user: '.$e->getMessage(),2);
+        }
+
+        try {
+            $sql = "INSERT INTO user (id,name,password,archive,role) VALUES (?,?,?,?,?)";
+            $sth = $DB->db->prepare($sql);
             $sth->bindValue(1,$this->id);
             $sth->bindValue(2,$this->name);
             $sth->bindValue(3,$this->password);
@@ -264,6 +332,46 @@ class UserDA extends Persistence
             $sth->bindValue(5,$this->role);
             $res = $sth->execute();
         }
+        catch(PDOException $e){ 
+            $App->log('WARN insert user: '.$e->getMessage(),2);
+        }
+        $DB->endTransaction();
+    }
+
+
+    // Overwrites Persistence::delete()
+    function delete ($attributes='')
+    {
+        global $App;
+ 
+        $DB = $App->DB;
+        $DB->startTransaction();
+        try {
+            $sql = 'DELETE FROM address WHERE id = ?';
+            $sth = $DB->db->prepare($sql);
+            $sth->bindValue(1,$this->id);
+            $res = $sth->execute();
+
+            $sql = 'DELETE FROM appstate WHERE userid = ?';
+            $sth = $DB->db->prepare($sql);
+            $sth->bindValue(1,$this->id);
+            $res = $sth->execute();
+
+            $sql = 'DELETE FROM container WHERE userid = ?';
+            $sth = $DB->db->prepare($sql);
+            $sth->bindValue(1,$this->id);
+            $res = $sth->execute();
+
+            $sql = 'DELETE FROM storedquery WHERE userid = ?';
+            $sth = $DB->db->prepare($sql);
+            $sth->bindValue(1,$this->id);
+            $res = $sth->execute();
+
+            $sql = 'DELETE FROM user WHERE id = ?';
+            $sth = $DB->db->prepare($sql);
+            $sth->bindValue(1,$this->id);
+            $res = $sth->execute();
+        }
         catch(PDOException $e){ 
             $App->log('WARN '.$e->getMessage(),0);
         }
@@ -322,16 +430,23 @@ class UserDA extends Persistence
             }
         }
 
-        if (!$this->role)    {
-            error_log("[User::external] ROLE not set by external auth");
-            throw new Exception('ERR_AUTH');
-        }
         if (!$this->archive) {
-            error_log("[User::external] ARCHIVE not set by external auth");
+            $App->log('[User::external] ARCHIVE not set by external auth',1);
             throw new Exception('ERR_ARCHIVE');
         }
         $this->archive = array_unique($this->archive);
 
+
+        if (!$this->role)    {
+            $App->log('[User::external] ROLE not set by external auth',1);
+            throw new Exception('ERR_AUTH');
+        }
+
+        if ($this->role == 'SYSTEM') { 
+            $App->log('[User::external] SYSTEM ROLE not allowed to login',1);
+            throw new Exception('ERR_AUTH');
+        }
+
         if ($this->role == 'ADMIN' || $this->role == 'REVISOR') {
             array_unshift($this->archive,'SYSDATA');
         }
@@ -347,5 +462,49 @@ class UserDA extends Persistence
 
         return $p;
     }
+
+
+    function _load_container()
+    {
+        global $App;
+        $entries = array();
+
+        $DB = $App->DB;
+        $sql = "SELECT * FROM container WHERE userid = ? ORDER BY cid,scid";
+        try {
+            $sth = $DB->db->prepare($sql);
+            $sth->bindValue(1,$this->id);
+            if($sth->execute()) {
+                while ($row = $sth->fetch()) {
+                      $cntid = $row['cid'];
+                      if ($row['scid'] != '') {
+                          $cntid.= '/'.$row['scid'];
+                      }
+                      array_push($this->archive,$cntid);
+                  }
+            }
+        }
+        catch(PDOException $e){ 
+            $App->log('WARN '.$e->getMessage(),0);
+        }
+
+        // deprecated
+        if (empty($this->archive)) {
+            $App->log('WARN: container table has no entries for '.$this->id);
+            $sql = "SELECT archive FROM user WHERE id = ?";
+            $sth = $DB->db->prepare($sql);
+            $sth->bindValue(1,$this->id);
+            if($sth->execute()) {
+                $row = $sth->fetch();
+                $cntlist = preg_split('/,\s+?/',$row['archive']);
+                foreach ($cntlist as $cntid) {
+                    if ($cntid) array_push($this->archive,$cntid);
+                }
+            }
+        }
+        // /deprecated
+    }
+
+
 }
 ?>
