@@ -50,6 +50,15 @@
 // $include_path = ini_get('include_path');
 // ini_set('include_path',$include_path.':lib');
 
+// ACCESS CONSTANTS
+define('BNO_AUTH',    0b00000010);   //   2 User authenticated
+define('BNO_USER',    0b00000100);   //   4 userpanel (change password)
+define('BNO_SYSINFO', 0b00001000);   //   8 Show system information
+define('BNO_SYSDATA', 0b00010000);   //  16 SYSDATA container
+define('BNO_ADMIN',   0b00100000);   //  32 admingui (change address, role)
+define('BNO_STATS',   0b01000000);   //  64 statistics
+define('BNO_SITEADM', 0b10000000);   // 128 admingui (change container)
+
 require_once 'model/User.php';
 require_once 'model/Backend.php';
 require_once 'model/Query.php';
